<?php

namespace App\Http\Controllers;

use App\Images;
use App\Models\BusinessContact;
use App\Models\BusinessContactDocument;
use Exception;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\File;

class BusinessContactController extends Controller
{
    public function index(Request $request){
        $search = [];
        
        if(isset($request->criteria)){
            $search = [                
                'criteria' => $request->criteria, 
                'keyword' => $request->keyword
            ];            
        }
                
        return view("business_contact.index", [            
            'search' => $search
        ]);
    }
    
    public function create(){
        return view("business_contact.create");
    }

    public function store(Request $request) {
        $request->validate([
            "lastname" => "required",
            "address" => "required",
            "phone" => "required",
            "file" => "required|image|mimes:jpeg,png,jpg,gif,svg|max:2048",
        ]); 
        
        try {
            DB::beginTransaction();

            $last = BusinessContact::whereNotNull("numero")->orderByDesc("created_at")->limit(1)->first();        
            $lastnum = !is_null($last) ? $last->numero : 0;
            $numero = $lastnum + 1;
            $reference = auto_number("business_contact", $numero);        

            $status = json_decode($request->status);        

            $business_contact = BusinessContact::create([
                "reference" => $reference,
                "lastname" => $request->lastname,
                "firstname" => $request->firstname,
                "address" => $request->address,
                "postcode" => $request->postcode,
                "city" => $request->city,
                "email" => $request->email,
                "phone" => $request->phone,
                "status" => $status->value,
                "userid" => Auth::user()->id,
                "username" => Auth::user()->name,
                "created_at" => date_now(),
                "updated_at" => date_now(),
            ]);

            $slug = $business_contact->id."-".strtolower(generateString(12));
            $business_contact->slug = $slug;
            
            if ($request->hasFile("file")) {
                $folder = "images/business_contacts/avatar";
                $datafile = $request->file;                

                if ($datafile->isValid()) {
                    $img = new Images($folder);
                    $rep = $img->image_upload($datafile, ['width' => '300px', 'height' => '300px']);
                    $business_contact->avatar = $rep['filename'];
                }
            }

            $business_contact->save();                    

            DB::commit();

            return response()->json([
                'status' => 'success',
                'message' => message()['success'],
                'detail' => [
                    'data' => $business_contact,
                    'url' => route('app.business_contact.detail', ['slug' => $slug])
                ]
            ]);
        } catch (\Throwable $th) {
            DB::rollBack();
            throw $th;
        }
    }
    
    public function detail(String $slug, Request $request){
        $res = BusinessContact::where('slug', $slug);

        switch ($request->tab) {
            case 'document':
                $business_contact = $res->with('documents')->first();
                break;            
            default:
                $business_contact = $res->first();
                break;
        }        

        return view("business_contact.detail", [
            'business_contact' => $business_contact
        ]);
    }

    public function info(Request $request) {
        try{
            $request->validate([
                "lastname" => "required"
            ]);
            $id = $request->id;
            BusinessContact::where("id", $id)->update([
                "lastname" => $request->lastname,
                "firstname" => $request->firstname,
                "address" => $request->address,
                "postcode" => $request->postcode,
                "city" => $request->city,
                "email" => $request->email,
                "phone" => $request->phone,
                "updated_at" => date_now()
            ]);

            return response()->json([
                'status' => 'success',
                'message' => message()['update'],
                'business_contact' => BusinessContact::find($id)
            ]);
        }catch(Exception $ex){
            throw $ex;
        }
    }

    public function memo(Request $request){
        try{            
            $request->validate([
                'memo' => 'required',
                'id' => 'required',
            ]);
            
            $id = $request->id;
            BusinessContact::where('id', $id)->update([
                "memo" => $request->memo,
                "updated_at" => date_now()
            ]);

            return response()->json([
                'status' => 'success',
                'message' => message()['update'],
                'business_contact' => BusinessContact::find($id)
            ]);
        }catch(Exception $ex){
            throw $ex;            
        }
    }

    public function destroy($id){
        BusinessContact::find($id)->delete();
        return response()->json([
            'status' => 'success',
            'message' => 'Revendeur supprimé avec succès'
        ]);
    }

    public function logo(Request $request){ 
        $request->validate([
            'file' => 'required'
        ]);        

        $id = $request->id;
        $business_contact = BusinessContact::find($id);
        
        if ($request->hasFile("file")) {
            $folder = "images/business_contacts/avatar";
            $datafile = $request->file;                

            if ($datafile->isValid()) {
                $img = new Images($folder);
                $rep = $img->image_upload($datafile, ['width' => '300px', 'height' => '300px']);
                $business_contact->avatar = $rep['filename'];
            }
        }
        $business_contact->save();

        return response()->json([
            'status' => 'success',
            'message' => message()['update'],
            'business_contact' => BusinessContact::find($id)
        ]);
    }

    public function documents(Request $request)
    {
        try {
            #dd($request->all());
            $request->validate([
                'title' => 'required',
                'file' => 'required'
            ]);

            DB::beginTransaction();

            $business_contact_id = $request->id;
            $business_contact = BusinessContact::find($business_contact_id);
            $doc = BusinessContactDocument::create([
                "business_contact_id" => $business_contact_id,
                "title" => $request->title,
                "comments" => $request->comments,
                "created_at" => date_now(),
                "updated_at" => date_now(),
            ]);            

            if ($request->hasFile("file")) {
                $folder = "documents/business_contacts/". $business_contact->slug;
                $datafile = $request->file;
                $fileType = $datafile->getMimeType();                

                if ($datafile->isValid()) {
                    $file = new Images($folder);
                    $rep = $file->move_file($datafile);                    
                    
                    BusinessContactDocument::where("id", $doc->id)->update([
                        'filename' => $rep['filename'],
                        'type' => $fileType,
                        'folder' => $business_contact->slug
                    ]);
                }
            }

            DB::commit();
            
            $business_contact = BusinessContact::find($business_contact_id)->with('documents')->first();
            return response()->json([
                'status' => 'success',
                'message' => message()['success'],
                'business_contact' => $business_contact
            ]);
        } catch (Exception $ex) {
            DB::rollBack();
            throw $ex;
        }
    }

    public function document_destroy($id){
        $doc = BusinessContactDocument::find($id);        
        
        $folder = asset('documents/business_contacts/'.$doc->folder.'/'.$doc->filename);
        if(File::exists($folder)){
            File::delete($folder);
        }
        $doc->delete();

        $business_contact_id = $doc->business_contact_id;
        $business_contact = BusinessContact::where('id', $business_contact_id)->with('documents')->first();
        
        return response()->json([
            'status' => 'success',
            'message' => 'Document supprimé avec succès',
            'business_contact' => $business_contact
        ]);
    }

    public function status(Request $request){        
        $id = $request->id;
        $status = $request->status['value'];
        $business_contact = BusinessContact::find($id);
        $business_contact->status = $status;
        $business_contact->save();

        return response()->json([
            'status' => 'success',
            'message' => message()['update'],
            'business_contact' => BusinessContact::find($id)
        ]);
    }

    public function business_contacts(Request $request){
        $res = BusinessContact::with('ventes', 'retours', 'commandes');
        
        $business_contacts = $res->paginate(25);
        return response()->json([
            'status' => 'success',
            'business_contacts' => $business_contacts
        ]);
    }


}
