<?php
namespace modules\sales\Services;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Session;
use modules\catalogs\services\ProductService;
use modules\catalogs\services\StockService;
use modules\crm\Models\BusinessContact;
use modules\crm\Models\BusinessContactTransaction;
use modules\location\Models\Location;
use modules\sales\Models\CaisseStatus;
use modules\sales\Models\CaisseStory;
use modules\sales\Models\CaisseTransaction;
use modules\sales\Models\Cart;
use modules\sales\Models\PaymentMethod;
use modules\sales\Models\Receipt;
use modules\sales\Models\ReceiptDetail;
use modules\sales\Models\ReceiptDiscount;
use modules\sales\Models\ReceiptPayment;

class ReceiptService {
    
    public function numero(){        
        $last = Receipt::where('status', '!=', 'draft')
                    ->whereNotNull("reference")
                    ->orderByDesc("created_at")
                    ->limit(1)->first();
        
        $lastnum = !is_null($last) ? $last->numero : 0;
        $index = $lastnum + 1;
        $num = auto_number('receipt', $index);
        return [
            "reference" => $num,
            "index" => $index
        ];
    }

    public function receipts(array $filters = [], int $page = 0)
    {
        $query = Receipt::with(['payments']);

        // Appliquer des filtres si nécessaires
        if (!empty($filters)) {
            // Applique tes filtres ici (exemple)
            
        }

        $query->orderByDesc('created_at');

        if ($page == 0) {
            $receipts = $query->get();
        } else {
            $receipts = $query->paginate($page);
        }

        return $receipts;
    }

    public function totals(array $filters = []) {
        $q = Receipt::where('status', '!=', 'cancel')->with(['payments']);
        return [
            'ht' => $q->sum('total_ht'),
            'ttc' => $q->sum('total_ttc'),
            'rap' => $q->get()->sum(function ($receipt) {
                return $receipt->total_ttc - $receipt->payments->sum('amount');
            }),
        ];
    }

    public function summarizeReceiptsByStatus() {
        $datastatus = Receipt::select('status')
            ->selectRaw('COUNT(*) as total, SUM(total_ht) as total_ht, SUM(total_ttc) as total_ttc')
            ->groupBy('status')
            ->get();

        $datastatus->prepend([
            'status' => 'all', 
            'total' => $datastatus->sum('total'),
            'total_ht' => $datastatus->sum('total_ht'),
            'total_ttc' => $datastatus->sum('total_ttc')
        ]);

        if (!$datastatus->contains('status', 'unpaid')) {
            $datastatus->push([
            'status' => 'unpaid',
            'total' => 0,
            'total_ht' => 0,
            'total_ttc' => 0
            ]);
        }

        if (!$datastatus->contains('status', 'paid')) {
            $datastatus->push([
            'status' => 'paid',
            'total' => 0,
            'total_ht' => 0,
            'total_ttc' => 0
            ]);
        }

        if (!$datastatus->contains('status', 'cancel')) {
            $datastatus->push([
            'status' => 'cancel',
            'total' => 0,
            'total_ht' => 0,
            'total_ttc' => 0
            ]);
        }

        return $datastatus;
    }

    public function processPayment(Request $request)
    {
        try {
            $this->validatePaymentRequest($request);
            
            $user = Auth::user();
            $customer = $request->customer;
            $amount = $request->amount;
            $token = $request->token;
            $paytoken = $request->paytoken;

            if(Session::has('paytoken') && $request->paytoken == Session::get('paytoken')){
                if ($this->isInsufficientAmount($customer, $amount)) {
                    return ['status' => 'error', 'message' => 'Montant insuffisant.', 'code' => 500];
                }
                
                DB::beginTransaction();
                
                $paymentMethod = PaymentMethod::find($request->payment_method);
                $ticket = Receipt::where('token', $token)->firstOrFail();
                $location = Location::find($ticket->location_id);
                
                if ($this->isPaymentAlreadyProcessed($paytoken)) {
                    return ['status' => 'error', 'message' => 'Token de paiement existant.', 'code' => 500];
                }
                
                $status = $amount['rap'] < 0 ? 'unpaid' : 'paid';
                $obs = $amount['rap'] < 0 ? 'Acompte' : 'Paiement';
                
                $caisseStatus = CaisseStatus::findOrFail(Session::get('caisse_open_id'));                
                
                $this->recordBusinessTransaction($customer, $ticket, $obs, 'out', $ticket->total_ttc);
                
                $sold = $caisseStatus->amount;
                if ((float) $amount['received'] > 0) {
                    $this->recordReceiptPayment($ticket, $caisseStatus, $user, $paymentMethod, $amount, $obs, $paytoken);
                    $this->recordBusinessTransaction($customer, $ticket, $obs, 'in', $amount['received']);
                    
                    if ((int) $paymentMethod->deposit_to_cashbox == 1) {
                        $sold = $this->getSold($caisseStatus->amount, $amount);
                        
                        $this->recordCashboxTransaction($caisseStatus, $customer, $user, $obs, $amount, $sold, $token);
                    }
                }                            
                
                $numero = $this->generateReceiptNumber();
                $ticket->update(['status' => $status, 'reference' => $numero['reference'], 'numero' => $numero['index']]);

                $this->updateStock($ticket, $location);
                $this->updateDiscountTable($ticket, $token);
                
                Cart::where('token', $token)->update(['status' => 'validated']);
                BusinessContactTransaction::where('receipt_id', $ticket->id)->update(['document_reference' => $numero['reference']]);
                
                DB::commit();
                
                return [
                    'status' => 'success', 
                    'sold' => $sold,
                    'location' => $location,
                    'ticket' => $ticket
                ];
            }else{
                return ['status' => 'error', 'message' => 'Token de paiement inexistant.', 'code' => 500];
            }                         
        } catch (\Throwable $th) {
            DB::rollBack();
            throw $th;
        }
    }
    
    private function validatePaymentRequest($request)
    {
        $request->validate(['token' => 'required']);
    }

    public function getSold($lastSold, $amount) {
        $r = max(0, (float) $amount['rap']);  
        return (float) $lastSold + ((float) $amount['received'] - $r);
    }    
    
    private function isInsufficientAmount($customer, $amount)
    {
        return (int) $customer['credit'] == 0 && $amount['rap'] < 0;
    }
    
    private function isPaymentAlreadyProcessed($paytoken)
    {
        return ReceiptPayment::where('token', $paytoken)->exists();
    }
    
    private function recordBusinessTransaction($customer, $ticket, $obs, $type, $amount)
    {        
        BusinessContactTransaction::create([
            'business_contact_id' => $customer['id'],
            'business_contact_name' => $customer['title'],
            'business_contact_type' => 'customer',
            'transaction_type' => $type,
            'document_type' => 'receipt',
            'receipt_id' => $ticket->id,
            'document_reference' => $ticket->reference,
            'document_amount' => $ticket->total_ttc,
            'description' => $obs,
            'amount' => $amount,
            'created_at' => now(),
            'updated_at' => now(),
        ]);
    }
    
    private function recordReceiptPayment($ticket, $caisseStatus, $user, $paymentMethod, $amount, $obs, $paytoken)
    {
        ReceiptPayment::create([
            "receipt_id" => $ticket->id,
            "caisse_status_id" => $caisseStatus->id,
            "caisse_id" => $caisseStatus->caisse_id,
            "caisse" => $caisseStatus->caisse,
            "reference" => $ticket->reference,
            "user_id" => $user->id,
            "username" => $user->name,
            "payment_method_id" => $paymentMethod->id,
            "payment_method" => $paymentMethod->title,
            "payment_additionnal" => json_encode(request()->additionnals),
            "amount" => $amount['received'] > $ticket->total_ttc ? $ticket->total_ttc : $amount['amount'],
            "observation" => $obs,
            "token" => $paytoken,
            "created_at" => now(),
            "updated_at" => now(),
        ]);
    }
    
    private function recordCashboxTransaction($caisseStatus, $customer, $user, $obs, $amount, $sold, $token)
    {
        
        CaisseStatus::where('id', Session::get('caisse_open_id'))->update([
            'amount' => $sold
        ]);
        
        CaisseStory::create([
            'caisse_status_id' => $caisseStatus->id,            
            'customer_id' => $customer['id'],
            'customer' => $customer['title'],
            'user_id' => $user->id,
            'user' => $user->name,
            'description' => $obs,
            'amount_in' => $amount['received'],
            'amount_out' => $amount['rap'] < 0 ? 0 : $amount['rap'],
            'sold' => $sold,
            'date_action' => date_now(),
            'token' => $token,
            'created_at' => date_now(),
            'updated_at' => date_now(),
        ]);
    }
    
    private function updateStock($ticket, $location)
    {
        $details = ReceiptDetail::where('token', $ticket->token)->get();
        foreach ($details as $detail) {
            (new ReceiptService())->handleStockDecrease($ticket->toArray(), $location, $detail);
        }
    }

    private function updateDiscountTable($ticket, $token)
    {
        ReceiptDiscount::where('token', $token)->update([
            'receipt_id' => $ticket->id
        ]);
    }
    
    private function generateReceiptNumber()
    {
        return (new ReceiptService())->numero();
    }

    public function handleStockDecrease(array $ticket, Location $location, $item)
    {
        $stockService = new StockService();
        $product = ProductService::products([
            'id' => $item->product_id,
            'location' => $location->id,
        ])->first();

        $stockActual = $product->stock[0]->stock ?? 0;
        $stockFinal = $stockActual - $item->quantity;        

        $stockService->update([
            'product_id' => $item->product_id,
            'location_id' => $location->id,
            'quantity' => $item->quantity,
        ], 'exit');

        $this->logStockMovement($stockService, $ticket, $location, $item, $stockActual, $stockFinal);        
    }

    private function logStockMovement($stockService, $ticket, $location, $item, $stockActual, $stockFinal)
    {
        $stockService->createUpdateControl([
            'product_id' => $item->product_id,
            'product_name' => $item->product,
            'location_id' => $location->id,
            'title' => 'VENTE #' . $ticket['reference'],
            'stock_actual' => $stockActual,
            'quantity' => $item->quantity,
            'stock_final' => $stockFinal,
            'type' => 'out',
        ]);

        $stockService->logStockStory([
            'product_id' => $item->product_id,
            'product_name' => $item->product,
            'location_id' => $location->id,
            'title' => 'VENTE #' . $ticket['reference'],
            'stock_actual' => $stockActual,
            'quantity' => $item->quantity,
            'stock_final' => $stockFinal,
            'token' => $ticket['token'],
        ]);
    }

}